using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;

namespace SlatedGameToolkit.Framework.Graphics
{
    internal delegate void GLClearColour(float r, float g, float b);
    internal delegate void GLClear(GLEnums flags);
    internal delegate void GLViewport(int x, int y, int width, int height);
    internal delegate UIntPtr GLCreateShader(GLEnums type);
    internal delegate UIntPtr GLShaderSource(UIntPtr shaderHandle, uint count, string program, int[] length);
    internal delegate void GLCompileShader(UIntPtr shaderHandle);
    internal delegate void GLGetShaderLogInfo(UIntPtr shader, uint maxLength, out uint writtenLength, out string output);
    internal delegate UIntPtr GLCreateProgram();
    internal delegate UIntPtr GLDeleteProgram(UIntPtr program);
    internal delegate void GLAttachShader(UIntPtr program, UIntPtr shader);
    internal delegate void GLDetachShader(UIntPtr program, UIntPtr shader);
    internal delegate void GLDeleteShader(UIntPtr shader);
    internal delegate void GLBindAttribLocation(UIntPtr program, uint index, string name);
    internal delegate void GLGetProgramInfoLog(UIntPtr program, uint maxLength, out uint writtenLength, out string output);
    internal delegate void GLLinkProgram(UIntPtr program);
    internal delegate void GLProgramParameter(UIntPtr program, GLEnums parameterName, int value);
    internal delegate void GLUseProgram(UIntPtr program);
    internal delegate void GLGenProgramPipelines(uint size, out UIntPtr[] pipelines);
    internal delegate void GLDeleteProgramPipelines(uint size, UIntPtr[] pipelines);
    internal delegate void GLBindProgramPipeline(UIntPtr pipeline);
    internal delegate void GLUseProgramStages(UIntPtr pipeline, GLEnums bitField, UIntPtr program);
    internal delegate uint GLGetError();
    internal delegate int GLGetAttribLocation(UIntPtr program, string attribName);
    internal delegate void GLBindBuffer(GLEnums target, UIntPtr buffer);
    internal delegate void GLGenBuffers(uint size, UIntPtr[] buffers);
    internal unsafe delegate void GLBufferData(GLEnums target, uint size, void* data, GLEnums usage);
    internal delegate void GLDeleteBuffers(uint size, UIntPtr[] buffers);
    internal delegate void GLVertexAttribPointer(uint index, int size, GLEnums type, bool normalized, uint stride, uint offset);
    internal delegate void GLGenVertexArrays(uint amount, UIntPtr[] arrays);
    internal delegate void GLDeleteVertexArrays(uint amount, UIntPtr[] arrays);    
    internal delegate void GLBindVertexArray(UIntPtr array);   
    internal delegate void GLEnableVertexAttribArray(uint attribIndex); 
    internal delegate void GLDisableVertexAttribArray(uint attribIndex);
    internal delegate void GLDrawArrays(GLEnums mode, int first, uint count);
    internal delegate void GLDrawElements(GLEnums mode, uint count, int offset);
    internal delegate void GLMultiDrawArrays(GLEnums mode, int[] first, uint[] count, uint primout);
    internal delegate void GLMultiDrawElements(GLEnums mode, uint[] count, GLEnums type, uint[] indices);
    internal delegate void GLGenTextures(uint n, UIntPtr[] textureHandles);
    internal delegate void GLBindTexture(GLEnums target, UIntPtr texture);
    internal delegate void GLDeleteTextures(uint n, UIntPtr[] textureHandles);
    internal delegate void GLTexParameteri(GLEnums target, GLEnums pname, int value);
    internal delegate void GLTexParameterf(GLEnums target, GLEnums pname, float value);
    internal delegate void GLTexParameteriv(GLEnums target, GLEnums pname, int[] values);
    internal delegate void GLTexParameterfv(GLEnums target, GLEnums pname, float[] values);
    internal delegate void GLTexImage2D(GLEnums target, int level, int internalFormat, uint width, uint height, int border, GLEnums format, GLEnums type, byte[] data);
    internal delegate void GLGenerateMipMap(GLEnums target);

    internal static class GLFunctionUtils {
        public static T RetrieveGLDelegate<T>(string functionName) where T : Delegate {
            IntPtr functionAddress = SDL.SDL_GL_GetProcAddress(functionName);
            if (functionAddress == null) throw new FrameworkSDLException();
            return Marshal.GetDelegateForFunctionPointer<T>(functionAddress);
        }
    }
}