using System;
using System.Drawing;
using System.Numerics;

namespace SlatedGameToolkit.Framework.Graphics.Meshes
{
    public class RectangleMesh : IMeshable
    {
        private bool changed = true;
        private Matrix4x4 matRot, matTrans, matScale;
        private Vector3 rotation, translation, scale;
        private Vector2 origin, dimensions;
        public readonly Vector2[] texCoords = new Vector2[4];
        private ValueTuple<Vector3, Vector2>[] vertices = new ValueTuple<Vector3, Vector2>[4];
        private uint[] indices = new uint[] {0, 1, 3, 1, 2, 3};

        public ValueTuple<Vector3, Vector2>[] Vertices
        {
            get {
                if (changed) CalculateVertices();
                return vertices;
            }
        }

        public float X {
            get 
            {
                return origin.X;
            }

            set 
            {
                origin.X = X;
            }
        }

        public float Y {
            get
            {
                return origin.Y;
            }

            set
            {
                origin.Y = value;
            }
        }

        public float Width
        {
            get
            {
                return dimensions.X;
            }

            set {
                dimensions.X = value;
            }
        }

        public float Height
        {
            get
            {
                return dimensions.Y;
            }

            set
            {
                dimensions.Y = value;
            }
        }

        public uint[] Elements { get {return indices; } }

        public Vector3 Translation 
        {
            get {
                return rotation; 
            }
            set {
                changed = true;
                translation = value;
                matTrans = Matrix4x4.CreateTranslation(value);
            }
        }

        public Vector3 Scale 
        {
            get {
                return scale;
            }

            set {
                changed = true;
                scale = value;
                matScale = Matrix4x4.CreateScale(value);
            }
        }

        public Vector3 Rotation 
        {
            get 
            {
                return rotation;
            }

            set
            {
                changed = true;
                rotation = value;
                matRot = Matrix4x4.CreateFromYawPitchRoll(value.X, value.Y, value.Z);
            }
        }
        private void CalculateVertices() {
            Vector3[] baseVerts = new Vector3[4];
            baseVerts[0] = new Vector3(this.origin, 0);
            baseVerts[1] = new Vector3(this.origin.X + this.dimensions.X, this.origin.Y, 0);
            baseVerts[2] = new Vector3(baseVerts[1].X, this.origin.Y + this.dimensions.Y, 0);
            baseVerts[3] = new Vector3(this.origin.X, baseVerts[2].Y, 0);

            Matrix4x4 transform = matTrans * matRot * matScale;
            for (int i = 0; i < vertices.Length; i++)
            {
                vertices[i] = new ValueTuple<Vector3, Vector2>(Vector3.Transform(baseVerts[i], transform), texCoords[i]);
            }
            changed = false;
        }
    }
}